--- asterisk-1.4.14/funcs/func_channel.c.orig	2007-11-20 16:46:32.000000000 -0500
+++ asterisk-1.4.14/funcs/func_channel.c	2007-11-20 16:47:19.000000000 -0500
@@ -16,9 +16,10 @@
 
 /*! \file
  *
- * \brief Channel info dialplan function
+ * \brief Channel info dialplan functions
  *
  * \author Kevin P. Fleming <kpfleming@digium.com>
+ * \author Ben Winslow
  * 
  */
 
@@ -30,6 +31,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <sys/types.h>
+#include <regex.h>
 
 #include "asterisk/module.h"
 #include "asterisk/channel.h"
@@ -189,14 +191,74 @@
 	.write = func_channel_write,
 };
 
+static int func_channels_read(struct ast_channel *chan, char *function, char *data, char *buf, size_t len)
+{
+	struct ast_channel *c = NULL, *old = NULL;
+	regex_t re;
+	int res;
+	
+	if (data != NULL && *data == '\0')
+		data = NULL;
+	
+	buf[0] = '\0';
+	
+	if (data != NULL) {
+		if ((res = regcomp(&re, data, REG_EXTENDED | REG_ICASE | REG_NOSUB))) {
+			regerror(res, &re, buf, len);
+			ast_log(LOG_WARNING, "Error compiling regular expression for %s(%s): %s\n", function, data, buf);
+			return -1;
+		}
+	}
+	
+	c = ast_channel_walk_locked(NULL);
+	
+	while (c) {
+		if (data == NULL || regexec(&re, c->name, 0, NULL, 0) == 0) {
+			if (strlen(buf) + strlen(c->name) + (buf[0] == '\0' ? 0 : 1) + 1 < len) {
+				if (buf[0] != '\0')
+					strcat(buf, " ");
+				strcat(buf, c->name);
+			}
+		}
+
+		old = c;
+		c = ast_channel_walk_locked(c);
+		ast_channel_unlock(old);
+	}
+	
+	if (data != NULL)
+		regfree(&re);
+	
+	return 0;
+}
+
+static struct ast_custom_function channels_function = {
+	.name = "CHANNELS",
+	.synopsis = "Gets the list of channels, optionally filtering by a regular expression.",
+	.syntax = "CHANNEL([regular expression])",
+	.desc = "Gets the list of channels, optionally filtering by a regular expression.\n"
+		"If no argument is provided, all known channels are returned.\n",
+	.read = func_channels_read,
+};
+
 static int unload_module(void)
 {
-	return ast_custom_function_unregister(&channel_function);
+	int res = 0;
+	
+	res |= ast_custom_function_unregister(&channel_function);
+	res |= ast_custom_function_unregister(&channels_function);
+	
+	return res;
 }
 
 static int load_module(void)
 {
-	return ast_custom_function_register(&channel_function);
+	int res = 0;
+	
+	res |= ast_custom_function_register(&channel_function);
+	res |= ast_custom_function_register(&channels_function);
+	
+	return res;
 }
 
-AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Channel information dialplan function");
+AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Channel information dialplan functions");
